@@ -5,18 +5,21 @@
  *   the Free Software Foundation; either version 2 of the License, or     *
  *   (at your option) any later version.                                   *
  *                                                                         *
- *  copyright (C) 2004-2006                                                *
+ *  copyright (C) 2004-2008                                                *
  *  Umbrello UML Modeller Authors <uml-devel@uml.sf.net>                   *
  ***************************************************************************/
 
 #ifndef MODEL_UTILS_H
 #define MODEL_UTILS_H
 
-#include <qstring.h>
-#include <qvaluelist.h>
+#include <QtCore/QString>
+#include <q3valuelist.h>
 
 #include "umlnamespace.h"
 #include "umlobjectlist.h"
+#include "foreignkeyconstraint.h"
+#include "icon_utils.h"
+
 
 /**
  * General purpose model utilities.
@@ -28,6 +31,8 @@
 // forward declarations
 class UMLClassifier;
 class UMLPackage;
+class UMLEntity;
+class UMLForeignKeyConstraint;
 
 namespace Model_Utils {
 
@@ -82,7 +87,7 @@ UMLObject* findUMLObject( const UMLObjectList& inList,
  */
 QString uniqObjectName(Uml::Object_Type type,
                        UMLPackage *parentPkg,
-                       QString prefix = QString::null);
+                       QString prefix = QString());
 
 /**
  * Return true if the given tag is a one of the common XMI
@@ -137,6 +142,11 @@ bool typeIsDiagram(Uml::ListView_Type type);
  */
 bool typeIsClassifierList(Uml::ListView_Type type);
 
+/**
+ * Return true if the listview type is a classifier ( Class, Entity , Enum )
+ */
+bool typeIsClassifier(Uml::ListView_Type type);
+
 /**
  * Return the Model_Type which corresponds to the given Diagram_Type.
  */
@@ -185,10 +195,10 @@ Uml::ListView_Type convert_OT_LVT(UMLObject *o);
  * Return the Icon_Type which corresponds to the given listview type.
  *
  * @param lvt  ListView_Type to convert.
- * @return  The Uml::Icon_Type corresponding to the lvt.
+ * @return  The Icon_Utils::Icon_Type corresponding to the lvt.
  *          Returns it_Home in case no mapping to Uml::Icon_Type exists.
  */
-Uml::Icon_Type convert_LVT_IT(Uml::ListView_Type lvt);
+Icon_Utils::Icon_Type convert_LVT_IT(Uml::ListView_Type lvt);
 
 /**
  * Return the Diagram_Type which corresponds to the given listview type.
@@ -233,6 +243,12 @@ QString progLangToString(Uml::Programming_Language pl);
  */
 Uml::Programming_Language stringToProgLang(QString str);
 
+
+/**
+ * Return string corresponding to Uml::Diagram_Type
+ */
+QString diagramTypeToString(Uml::Diagram_Type dt);
+
 /**
  * Return type of parseOperation()
  */
@@ -253,7 +269,7 @@ struct NameAndType {
     }
     NameAndType(QString name, UMLObject *type,
                 Uml::Parameter_Direction direction = Uml::pd_In,
-                QString initialValue = QString::null)
+                QString initialValue = QString())
             : m_name(name), m_type(type),
               m_direction(direction), m_initialValue(initialValue) {
     }
@@ -262,8 +278,8 @@ struct NameAndType {
 /**
  * Auxiliary type for OpDescriptor
  */
-typedef QValueList<NameAndType> NameAndType_List;
-typedef QValueListIterator<NameAndType> NameAndType_ListIt;
+typedef Q3ValueList<NameAndType> NameAndType_List;
+typedef Q3ValueListIterator<NameAndType> NameAndType_ListIt;
 
 /**
  * Data structure filled by parseOperation()
@@ -293,9 +309,18 @@ Parse_Status parseTemplate(QString t, NameAndType& nmTp, UMLClassifier *owningSc
  *                      Example:  argname : argtype
  * @param nmTp          NameAndType returned by this method.
  * @param owningScope   Pointer to the owning scope of the attribute.
+ * @param vis           Optional pointer to visibility (return value.)
+ *                      The visibility may be given at the beginning of the
+ *                      attribute text in mnemonic form as follows:
+ *                      "+"  stands for public
+ *                      "#"  stands for protected
+ *                      "-"  stands for private
+ *                      "~"  stands for implementation level visibility
+ *
  * @return      Error status of the parse, PS_OK for success.
  */
-Parse_Status parseAttribute(QString a, NameAndType& nmTp, UMLClassifier *owningScope);
+Parse_Status parseAttribute(QString a, NameAndType& nmTp, UMLClassifier *owningScope,
+                            Uml::Visibility *vis = 0);
 
 /**
  * Parses an operation given in UML syntax.
@@ -309,11 +334,29 @@ Parse_Status parseAttribute(QString a, NameAndType& nmTp, UMLClassifier *owningS
  */
 Parse_Status parseOperation(QString m, OpDescriptor& desc, UMLClassifier *owningScope);
 
+/**
+ * Parses a constraint
+ *
+ * @param m             Input text of the constraint
+ *
+ * @param name          The name returned by this method
+ * @param owningScope   Pointer to the owning scope of the constraint
+ * @return       Error status of the parse, PS_OK for success.
+ */
+Parse_Status parseConstraint(QString m, QString& name, UMLEntity* owningScope);
+
+
 /**
  * Returns the Parse_Status as a text.
  */
 QString psText(Parse_Status value);
 
+/**
+ * Converts from the UpdateDeleteAction enum to a QString
+ * @param uda The UpdateDeleteAction enum literal
+ */
+QString updateDeleteActionToString( UMLForeignKeyConstraint::UpdateDeleteAction uda );
+
 }
 
 #endif
