@@ -1,29 +1,21 @@
 /***************************************************************************
- *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
  *   the Free Software Foundation; either version 2 of the License, or     *
  *   (at your option) any later version.                                   *
  *                                                                         *
- *   copyright (C) 2002-2006                                               *
+ *   copyright (C) 2002-2008                                               *
  *   Umbrello UML Modeller Authors <uml-devel@uml.sf.net>                  *
  ***************************************************************************/
 
 #ifndef UML_H
 #define UML_H
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
 #include "umlnamespace.h"
 
-#include <qmap.h>
-#include <qdict.h>
-
-#include <kdockwidget.h>
-#include <kdeversion.h>
+#include <kxmlguiwindow.h>
 #include <kurl.h>
+#include <ksharedconfig.h>
 
 // forward declaration of the UML classes
 class AlignToolBar;
@@ -46,27 +38,36 @@ class XhtmlGenerator;
 // KDE forward declarations
 class KActionMenu;
 class KRecentFilesAction;
-class KStatusBarLabel;
 class KToggleAction;
-class KDockWidget;
 class KTabWidget;
-class KToolBarButton;
-class KPopupMenu;
+class KMenu;
+class KMenuBar;
+class KUndoStack;
+class KAction;
 
 // Qt forward declarations
-class QWidgetStack;
-class QMenuData;
-class QClipboard;
+class QStackedWidget;
 class QToolButton;
 class QCustomEvent;
+class QDockWidget;
+class QVBoxLayout;
+class QKeyEvent;
+class QMenu;
+class QMimeData;
+class QUndoCommand;
+class QUndoView;
 
 /**
  * The base class for UML application windows. It sets up the main
  * window and reads the config file as well as providing a menubar, toolbar
- * and statusbar. An instance of UMLView creates your center view, which is connected
- * to the window's Doc object.
+ * and statusbar. A list of UMLView instances creates the center views, which are connected
+ * to the window's Doc object. The handling of views is realized with two different widgets:
+ *   - stack widget
+ *   - tab widget
+ * The current view handling is set as an option. 
  * UMLApp reimplements the methods that KMainWindow provides for main window handling and supports
  * full session management as well as using KActions.
+ * 
  * @see KMainWindow
  * @see KApplication
  * @see KConfig
@@ -75,25 +76,29 @@ class QCustomEvent;
  * Bugs and comments to uml-devel@lists.sf.net or http://bugs.kde.org
  */
 
-class UMLApp : public KDockMainWindow {
+class UMLApp : public KXmlGuiWindow
+{
     Q_OBJECT
 public:
     /**
      * Constructor. Calls all init functions to create the application.
      */
-    UMLApp(QWidget* parent=0, const char* name=0);
+    UMLApp(QWidget* parent=0);
 
     /**
      * Standard deconstructor.
      */
     ~UMLApp();
 
+    /**
+     * Get the last created instance of this class.
+     */
     static UMLApp* app();
 
     /**
      * Opens a file specified by commandline option.
      */
-    void openDocumentFile(const KURL& url=KURL());
+    void openDocumentFile(const KUrl& url=KUrl());
 
     /**
      * Calls the UMLDoc method to create a new Document.
@@ -197,14 +202,21 @@ public:
      */
     CodeGenerator* getGenerator();
 
+    /**
+     * Determines if SimpleCodeGenerator is active.
+     *
+     * @return  true if SimpleCodeGenerator is active.
+     */
+    bool isSimpleCodeGeneratorActive();
+
     /**
      * Set the current generator for this app.
      * If giveWarning is true, then a popup box warning that the
      * code generation library is out-of-date will show if you
      * attempt to set the generator to NULL.
      *
-     * @param gen               Pointer to the CodeGenerator to set.
-     * @param giveWarning       True to enable out-of-date warning.
+     * @param gen           Pointer to the CodeGenerator to set.
+     * @param giveWarning   True to enable out-of-date warning.
      */
     void setGenerator(CodeGenerator* gen, bool giveWarning = true);
 
@@ -255,7 +267,7 @@ public:
      * Puts this view to the top of the viewStack, i.e. makes it
      * visible to the user.
      *
-     * @param view              Pointer to the UMLView to push.
+     * @param view   Pointer to the UMLView to push.
      */
     void setCurrentView(UMLView* view);
 
@@ -273,7 +285,7 @@ public:
      *
      * @param mimeType  The MIME type to set as the default.
      */
-    void setImageMimeType(QString const & mimeType){m_imageMimeType=mimeType;};
+    void setImageMimeType(QString const & mimeType){m_imageMimeType=mimeType;}
 
     /**
      * Gets the default mime type for all diagrams that are exported as
@@ -281,7 +293,7 @@ public:
      *
      * @return  The default MIME type for images.
      */
-    QString const & getImageMimeType()const{return m_imageMimeType;};
+    QString const & getImageMimeType()const{return m_imageMimeType;}
 
     /**
      * Carries out the cut/copy command with different action performed
@@ -321,6 +333,37 @@ public:
      */
     CodeGenPolicyExt *getPolicyExt();
 
+    /**
+     * Removes all entries from the UndoStack and RedoStack and disables the
+     * undo and redo actions.
+     */
+    void clearUndoStack();
+
+    /**
+     * Undo last command
+     */
+    void undo();
+
+    /**
+     * Redo last 'undoed' command
+     */
+    void redo();
+
+    /**
+     * Execute a command and pushit in the stack.
+     */
+    void executeCommand(QUndoCommand* cmd);
+
+    /**
+     * Begin a U/R command macro
+     */
+    void BeginMacro( const QString & text );
+
+    /**
+     * End an U/R command macro
+     */
+    void EndMacro();
+
 protected:
     virtual void keyPressEvent(QKeyEvent* e);
     virtual void keyReleaseEvent(QKeyEvent* e);
@@ -330,7 +373,7 @@ protected:
      * It handles events such as exporting all views from command line (in
      * that case, it executes the exportAllViews method in the event).
      */
-    virtual void customEvent(QCustomEvent* e);
+    virtual void customEvent(QEvent* e);
 
     /**
      * Helper method for handling cursor key release events (refactoring).
@@ -351,7 +394,8 @@ protected:
     void readOptions();
 
     /**
-     * Initializes the KActions of the application.
+     * Initializes the KActions and the status bar of the application 
+     * and calls setupGUI().
      */
     void initActions();
 
@@ -401,7 +445,7 @@ protected:
      * temporary filename provided by KApplication.
      * @see KMainWindow#saveProperties
      */
-    virtual void saveProperties(KConfig *_cfg);
+    virtual void saveProperties(KConfigGroup & cfg);
 
     /**
      * Reads the session config file and restores the
@@ -410,7 +454,7 @@ protected:
      * saveProperties()
      * @see KMainWindow#readProperties
      */
-    virtual void readProperties(KConfig *_cfg);
+    virtual void readProperties(const KConfigGroup & cfg);
 
     CodeGenerationPolicy * m_commoncodegenpolicy;
 
@@ -422,13 +466,6 @@ protected:
      */
     void updateLangSelectMenu(Uml::Programming_Language activeLanguage);
 
-protected slots:
-
-    /**
-     * Show "Tip of the Day" dialog
-     */
-    void tipOfTheDay();
-
 public slots:
 
     /**
@@ -455,7 +492,7 @@ public slots:
     /**
      * Opens a file from the recent files menu.
      */
-    void slotFileOpenRecent(const KURL& url);
+    void slotFileOpenRecent(const KUrl& url);
 
     /**
      * Save a document.
@@ -488,11 +525,11 @@ public slots:
     void slotFileQuit();
 
      /**
-      * Exports the current model to docbook in a subdir of the
-      * current model directory named from the model name.
-      * @todo Let the user chose the destination directory and
-      * name, using network transparency.
-      */
+     * Exports the current model to docbook in a subdir of the
+     * current model directory named from the model name.
+     * @todo Let the user chose the destination directory and
+     * name, using network transparency.
+     */
     void slotFileExportDocbook();
 
     /**
@@ -519,27 +556,10 @@ public slots:
      */
     void slotEditPaste();
 
-    /**
-     * Toggles the toolbar.
-     * Deprecated. For compatibility with KDE 3.1, remove if we stop supporting KDE 3.1
-     */
-    void slotViewToolBar();
-
-    /**
-     * Toggles the statusbar.
-     * Deprecated. For compatibility with KDE 3.1, remove if we stop supporting KDE 3.1
-     */
-    void slotViewStatusBar();
-
-    /**
-     * Autolayouts the current class diagram
-     */
-    void slotAutolayout();
-
     /**
      * Changes the statusbar contents for the standard label
      * permanently, used to indicate current actions.
-     * @param text      The text that is displayed in the statusbar
+     * @param text   The text that is displayed in the statusbar
      */
     void slotStatusMsg(const QString &text);
 
@@ -582,18 +602,59 @@ public slots:
      * Create this view.
      */
     void slotDeploymentDiagram();
+
     /**
      * Create this view.
      */
     void slotEntityRelationshipDiagram();
 
+    /**
+     * Left Alignment
+     */
+    void slotAlignLeft();
+
+    /**
+     * Right Alignment
+     */
+    void slotAlignRight();
+
+    /**
+     * Top Alignment
+     */
+    void slotAlignTop();
+
+    /**
+     * Bottom Alignment
+     */
+    void slotAlignBottom();
+
+    /**
+     * Vertical Middle Alignment
+     */
+    void slotAlignVerticalMiddle();
+
+    /**
+     * Horizontal Middle Alignment
+     */
+    void slotAlignHorizontalMiddle();
+
+    /**
+     * Vertical Distribute Alignment
+     */
+    void slotAlignVerticalDistribute();
+
+    /**
+     * Horizontal Distribute Alignment
+     */
+    void slotAlignHorizontalDistribute();
+
     /**
      * Notification of changed clipboard data.
      */
     void slotClipDataChanged();
 
     /**
-     *
+     * Slot for enabling cut and copy to clipboard.
      */
     void slotCopyChanged();
 
@@ -621,17 +682,9 @@ public slots:
     /**
      * Set the language for which code will be generated.
      *
-     * @param menuID    the ID of the langSelect menu item for
-     *                  the relevant language.
+     * @param pl    The name of the language to set
      */
-    void setActiveLanguage(int menuID);
-
-    /**
-     * Set the language for which code will be generated.
-     *
-     * @param activeLanguage The name of the language to set
-     */
-    void setActiveLanguage( const QString &activeLanguage );
+    void setActiveLanguage(Uml::Programming_Language pl);
 
     /**
      * Get the language for import and code generation.
@@ -690,13 +743,18 @@ public slots:
      */
     void slotImportClasses();
 
+    /**
+     * Import project menu selection.
+     */
+    void slotImportProject();
+
     /**
      * Class wizard menu selection.
      */
     void slotClassWizard();
 
     /**
-     * Calls the active code generator to add its default datatypes
+     * Calls the active code generator to add its default datatypes.
      */
     void slotAddDefaultDatatypes();
 
@@ -727,26 +785,33 @@ public slots:
 
     /**
      * Deletes the current diagram.
+     * @param tab   Widget's tab to close
      */
-    void slotDeleteDiagram();
+    void slotDeleteDiagram(QWidget* tab = NULL);
 
     /**
      * Set the zoom factor of the current diagram.
      *
-     * @param zoom              Zoom factor in percentage.
+     * @param zoom  Zoom factor in percentage.
      */
     void setZoom(int zoom);
 
+    /**
+     * Set the zoom factor of the current diagram.
+     *
+     * @param action  Action which is called.
+     */
+    void slotSetZoom(QAction* action);
+
     /**
      * Connected to by the KPlayerSliderAction zoomAction, a value of between 300
      * and 2200 is scaled to zoom to between 9% and 525%.
      * The min and max values of the slider are hard coded in KPlayerSliderAction for now.
-     * @param value         Zoom factor before scaleing
      */
     void slotZoomSliderMoved(int value);
 
     /**
-     * Set zoom to 100%
+     * Set zoom to 100%.
      */
     void slotZoom100();
 
@@ -768,58 +833,105 @@ public slots:
     void slotEditRedo();
 
     /**
-     * Searches for a menu with the given name
+     * Searches for a menu with the given name.
      *
-     * @param menu      The QPopupMenu or QMenuBar to search through.
-     * @param name      The name of the menu to search for (name, not text)
+     * @param name  The name of the menu to search for (name, not text)
      */
-    QPopupMenu* findMenu(QMenuData* menu, const QString &name);
+    QMenu* findMenu(const QString &name);
 
     /**
-     * called when the tab has changed
+     * Called when the tab has changed.
+     * @param tab   The changed tab widget
      */
-    void slotTabChanged(QWidget* view);
+    void slotTabChanged(QWidget* tab);
 
     /**
-     * make the tab on the left of the current one the active one
+     * Make the tab on the left of the current one the active one.
      */
     void slotChangeTabLeft();
 
     /**
-     * make the tab on the right of the current one the active one
+     * Make the tab on the right of the current one the active one.
      */
     void slotChangeTabRight();
 
     /**
-     * Move the current tab left, not implemented
+     * Move the current tab left.
      */
     void slotMoveTabLeft();
 
     /**
-     * Move the current tab right, not implemented
+     * Move the current tab right.
      */
     void slotMoveTabRight();
 
-    KConfig *getConfig() { return m_config; }
+    /**
+     * Get the configuration data.
+     */
+    KConfig* getConfig() { return m_config.data(); }
 
     /**
      * This slot deletes the current XHTML documentation generator as soon as
      * this one signals that it has finished.
+     * @param status true if successful else false
      */
-    void slotXhtmlDocGenerationFinished();
+    void slotXhtmlDocGenerationFinished(bool status);
+
+private slots:
+
+    /**
+     * Slots for connection to the QActions of the m_langSelect menu.
+     */
+    void setLang_actionscript();
+    void setLang_ada();
+    void setLang_cpp();
+    void setLang_csharp();
+    void setLang_d();
+    void setLang_idl();
+    void setLang_java();
+    void setLang_javascript();
+    void setLang_mysql();
+    void setLang_pascal();
+    void setLang_perl();
+    void setLang_php();
+    void setLang_php5();
+    void setLang_postgresql();
+    void setLang_python();
+    void setLang_ruby();
+    void setLang_sql();
+    void setLang_tcl();
+    void setLang_xmlschema();
 
 private:
+    /**
+     * Variable for holding the last created instance of this class.
+     */
     static UMLApp* s_instance;
 
     /**
      * For selecting the active language.
      */
-    QPopupMenu *m_langSelect;
+    QMenu *m_langSelect;
 
     /**
-    * Popup menu for zoom selection.
-    */
-    QPopupMenu *m_zoomSelect;
+     * Popup menu for zoom selection.
+     */
+    QMenu *m_zoomSelect;
+
+    /**
+     * Helper method to create the zoom actions.
+     */
+    QAction* createZoomAction(int zoom, int currentZoom);
+
+    /**
+     * Helper method to reset the status bar message.
+     */
+    void resetStatusMsg();
+
+    /**
+     * Helper method to setup the programming language action.
+     */
+    void setProgLangAction(Uml::Programming_Language pl, const QString& name, const QString& action);
 
     /**
      * Active language.
@@ -833,14 +945,14 @@ private:
 
     /**
      * Active policy extension.
-     * Only used for new code generators ({Cpp,Java,Ruby}CodeGenerator)
+     * Only used for new code generators ({Cpp,Java,Ruby,D}CodeGenerator)
      */
     CodeGenPolicyExt *m_policyext;
 
     /**
      *  Returns whether we can decode the given mimesource
      */
-    static bool canDecode(const QMimeSource* mimeSource);
+    static bool canDecode(const QMimeData* mimeSource);
 
     /**
      * Reads from the config file the options state.
@@ -863,9 +975,14 @@ private:
     void initSavedCodeGenerators();
 
     /**
-     * The configuration object of the application.
+     * import the source files that are in fileList
      */
-    KConfig* m_config;
+    void importFiles(QStringList* fileList);
+
+     /**
+      * The configuration object of the application.
+      */
+    KSharedConfigPtr m_config;
 
     /**
      * View is the main widget which represents your working area.
@@ -890,84 +1007,67 @@ private:
     /**
      * The widget which shows the diagrams.
      */
-    KDockWidget* m_mainDock;
+    QDockWidget* m_mainDock;
 
     /**
      * Contains the UMLListView tree view.
      */
-    KDockWidget* m_listDock;
+    QDockWidget* m_listDock;
 
     /**
      * Contains the documentation DocWindow widget.
      */
-    KDockWidget* m_documentationDock;
+    QDockWidget* m_documentationDock;
+
+    /**
+     * Contains the undo/redo viewer widget.
+     */
+    QDockWidget* m_cmdHistoryDock;
+
+    /**
+     * Contains the property browser widget
+     */
+    QDockWidget* m_propertyDock;
 
     /**
      * Documentation window.
      */
     DocWindow* m_pDocWindow;
 
+    /**
+     * Undo / Redo Viewer
+     */
+    QUndoView* m_pQUndoView;
+
     /** Refactoring assistant. */
     RefactoringAssistant* m_refactoringAssist;
 
-    //KAction pointers to enable/disable actions
-    KAction* fileNew;
-    KAction* fileOpen;
+    // KAction pointers to enable/disable actions
     KRecentFilesAction* fileOpenRecent;
-    KAction* fileSave;
-    KAction* fileSaveAs;
-    KAction* fileClose;
-    KAction* filePrint;
-    KAction* fileQuit;
-    KAction* fileExportDocbook;
-    KAction* fileExportXhtml;
-
-    KAction* editCut;
-    KAction* editCopy;
-    KAction* editPaste;
-    KAction* editUndo;
-    KAction* editRedo;
-    KAction* selectAll;
-    KAction* preferences;
+    QAction* filePrint;
+    QAction* editCut;
+    QAction* editCopy;
+    QAction* editPaste;
+    QAction* editUndo;
+    QAction* editRedo;
 
     KActionMenu* newDiagram;
-    KAction* classDiagram;
-    KAction* sequenceDiagram;
-    KAction* collaborationDiagram;
-    KAction* useCaseDiagram;
-    KAction* stateDiagram;
-    KAction* activityDiagram;
-    KAction* componentDiagram;
-    KAction* deploymentDiagram;
-    KAction* entityRelationshipDiagram;
-    KAction* viewClearDiagram;
+    QAction* viewClearDiagram;
 
     KToggleAction* viewSnapToGrid;
     KToggleAction* viewShowGrid;
-    KAction* viewExportImage;
-    KAction* viewExportImageAll;
-    KAction* viewProperties;
+    QAction* viewExportImage;
+    QAction* viewProperties;
 
-    KAction* zoom100Action;
+    QAction* zoom100Action;
     KPlayerPopupSliderAction* zoomAction;
 
-    KAction* genAll;
-    KAction* genWizard;
-    KAction* importClasses;
-    KAction* classWizard;
+    QAction* m_langAct[Uml::pl_Reserved];
     KAction* deleteSelectedWidget;
     KAction* deleteDiagram;
-#ifdef HAVE_DOT
-    KAction* autolayout;
-#endif
 
-    KAction* changeTabLeft;
-    KAction* changeTabRight;
-    KAction* moveTabLeft;
-    KAction* moveTabRight;
-    KToolBarButton* m_newSessionButton;
-    KPopupMenu* m_diagramMenu;
-    QToolButton* m_closeDiagramButton;
+    QToolButton* m_newSessionButton;
+    KMenu* m_diagramMenu;
     KToggleAction* viewToolBar;
     KToggleAction* viewStatusBar;
     WorkToolBar* toolsbar;
@@ -975,8 +1075,6 @@ private:
     QTimer* m_copyTimer;
     AlignToolBar* m_alignToolBar;
 
-    KStatusBarLabel* m_statusLabel;
-
     /**
      * True if the application is opening an existing document
      */
@@ -986,7 +1084,7 @@ private:
      * Shows, and is parent of, all the UMLViews (diagrams)
      * if tabbed diagrams are not enabled.
      */
-    QWidgetStack* m_viewStack;
+    QStackedWidget* m_viewStack;
 
     /**
      * Shows, and is parent of, all the UMLViews (diagrams)
@@ -994,6 +1092,13 @@ private:
      */
     KTabWidget* m_tabWidget;
 
+    /**
+     * Layout supports the dynamic management of the diagram representation (tabbed/stacked)
+     * if tabbed diagrams is enabled it contains m_tabWidget
+     * if tabbed diagrams is disabled it contains m_viewStack
+     */
+    QVBoxLayout* m_layout;
+
     /**
      * Default mime type to use for image export.
      */
@@ -1015,6 +1120,17 @@ private:
      */
     XhtmlGenerator* m_xhtmlGenerator;
 
+    /**
+     * UndoStack
+     * used to store actions, to provide Undo/Redo feature.
+     */
+    KUndoStack* m_pUndoStack;
+
+    /**
+     * Macro creation flag
+     */
+    bool m_hasBegunMacro;
+
 signals:
 
     /**
